/* SPDX-License-Identifier: (GPL-2.0 OR BSD-3-Clause) */
/*
 * Copyright (c) 2020 MediaTek Inc.
 */

#ifndef MDRV_DEMUX_TEE_COMM_H
#define MDRV_DEMUX_TEE_COMM_H

int mdrv_demux_tee_open_ctx(void);
int mdrv_demux_tee_close_ctx(void);
int mdrv_demux_tee_open_session(u32 *session_id, uuid_t uuid);
int mdrv_demux_tee_close_session(u32 session_id);
int mdrv_demux_tee_invoke_cmd(u32 session_id, u32 cmd, void *param, size_t size);

#endif /* MDRV_DEMUX_TEE_COMM_H */
